<?php
/**
 * Example to show how to use MBS Lib. 
 * Show 
 * 	- Menu
 *  - Show items to buy
 *  - Generate buy link
 *  - Display Bought
 *  - Commit transaction 
 *  - Rallback transaction
 *  - Show list of transactions with information about them
 *  
 * Example class
 * @package ExampleProject
 * @subpackage module
 * 
 * @author Liutauras Rickus <lr@vero.lt>
 *
 */

/**
 * ModuleMbs
 * 
 * @package ExampleProject
 * @subpackage module
 * @author Liutauras Rickus <lr@vero.lt>
 */
class ModuleMbs {
	
	/**
	 * Internal MBS object
	 *
	 * @var MBS
	 */
	var $mbs = null;
	
	/**
	 * This action
	 *
	 * @var array
	 */
	var $actions = '';
		
	/**
	 * Internal data for show how to use
	 *
	 * @var array
	 */
	var $items = array();
	
	/**
	 * Parameter 
	 *
	 * @var unknown_type
	 */
	var $params_item = 'letters';
	
	/**
	 * Transaction Info
	 *
	 * @var array
	 */
	var $transaction_info = array();
	
	/**
	 * Internal used data
	 *
	 * @var mixed
	 */
	var $data = '';
		
	/**
	 * Constructor
	 *
	 * @param MBS $mbs - from lib
	 * @return ModuleMbs
	 */
	function ModuleMbs( &$mbs ) {
		$this->mbs = &$mbs;
		$this->actions = explode('.', $_REQUEST['act']);
		
		$this->items = array( 'a' => 'alpha', 'b' => 'beta', 'g' => 'gama', 'd' => '', 'e' => 'epsilon',  );
	}
	
	/**
	 * Main action for module
	 *
	 */
	function parseAction() {
		if ($this->actions[0] != 'mbs') {
			return false;
		}
		
		switch ($this->actions[1]) {
			// Making billing action
            case 'buy_redirect':
            	// Save params in array
            	$params = array(
                    'act'       => 'mbs.donate', 
                    'session_id'=> session_id(), 
            	);
            	// Make billing
            	$this->mbs->billRedirect($params, null, $_GET['price'], $_GET['currency']);
            	break;
			
			// Making billing action
			case 'buy':
				// Save params in array
				$params = array(
					$this->params_item 	=> $_GET[$this->params_item], 
					'act'			 	=> 'mbs.download',
					'session_id'        => session_id(),
					't'					=> '%t',		// set transaction
					'a'					=> '%a',		// set account_id
					's'					=> '%s',		// set service_id
					'o'					=> '%o',		// set operator id
				);
				// Make billing
				$this->mbs->bill($params, $_SESSION['lang']);
				break;
			case 'list':
				// Get one transaction info
				if (!empty($_GET['trans']) ) {
					$this->transaction_info = $this->mbs->getTransaction($_GET['trans']);
				}
				break;
			// Billing done now check billing status
			case 'download':
				if (!empty($_GET['t'])) {
					// Checking is billing is OK
					if ($this->mbs->isBilled($_GET['t']) && !$this->mbs->isCommited($_GET['t'])) {
						// If user got service need to commit transaction.
						// Else grant service to user and commit transaction.
						// $this->mbs->commitTransaction($_GET['transaction_id']);
						$_SESSION['letters_status'][$_GET[$this->params_item]] = 'paid';
						
					} else {
						$this->actions[1] = 'letters';
					}
					$this->transaction_info = $this->mbs->getTransaction($_GET['t']);
					$this->transaction_info['params'] = $_GET;
					// Save used transaction in session
					$_SESSION['transactions'][$_GET['t']] = $_GET['t'];
				}
				break;
			// After item download
			case 'done':
				// Get transaction status
				$this->transaction_info = $this->mbs->getTransaction($_GET['transaction_id']);
				// If transaction paid commit transaction
				if ($this->mbs->isBilled($_GET['transaction_id'])) {
					$commit = $this->mbs->commitTransaction($_GET['transaction_id']);
					// If commited save status of transaction
					if ($commit && !$this->mbs->isError()) {
						$_SESSION['letters_status'][$_GET[$this->params_item]] = 'downloaded';
					}
				} elseif( $this->mbs->isCommited($_GET['transaction_id']) ) {
					$_SESSION['letters_status'][$_GET[$this->params_item]] = 'commited';
				}
				$this->transaction_info = $this->mbs->getTransaction($_GET['transaction_id']);
				$this->actions[1] = $this->params_item;
				break;
			// If user can't download rallback transaction
			case 'error':
				// Get status
				$this->transaction_info = $this->mbs->getTransaction($_GET['transaction_id']);
				// If transaction paid rollback it
				if ($this->mbs->isBilled($_GET['transaction_id'])) {
					$rollback = $this->mbs->rollBackTransaction($_GET['transaction_id']);
					if ($rollback && !$this->mbs->isError()) {
						$_SESSION['letters_status'][$_GET[$this->params_item]] = 'rollbacked';
					}
				}
				$this->transaction_info = $this->mbs->getTransaction($_GET['transaction_id']);
				$this->actions[1] = $this->params_item;
				break;
			default:
		}
		
		return true;
	}
	
	/**
	 * Show elements from $this->items(like shop)
	 *
	 */
	function showElements() {
		if ( $this->actions[0] == 'mbs' ) {
			foreach ($this->items as $key => $item) {
				?>
				<?php echo $key;?> 
				<a href="?act=mbs.buy&<?php echo $this->params_item?>=<?php echo $key?>"><?php lang('buy')?></a>
				<?php echo $_SESSION['letters_status'][$key];?> 
				<br />
				<?php
			}
		}
		if (!empty($this->transaction_info)) echo $this->mbs->getStatusDescription($this->transaction_info['state']);
	}
	
	/**
	 * Show transaction information from $this->transaction_info
	 *
	 */
	function showTransactions() {
		if (empty($_SESSION['transactions'])) { $_SESSION['transactions'] = array(); }
		foreach ($_SESSION['transactions'] as $item) {
			?>
		<?php echo $item?> - <a href="?act=mbs.list&trans=<?php echo $item?>" class="<?php echo (($_GET['trans'] == $item) ? 'selected' : '' )?>"><?php lang('detail');?></a><br />
			<?php
		}
		if (!empty($this->transaction_info)) {
			echo $this->mbs->getStatusDescription($this->transaction_info['state'])."<br>\n";
			echo '<pre>';
			print_r($this->transaction_info);
			echo '</pre>';
		}
		
	}
	
	
	/**
	 * Main display for module. It must be change by template.
	 *  - Play items list
	 *  - Download item
	 *  - Show transactions info
	 *
	 */
	function showBlock() {
		foreach ($this->actions as $key => $item) {
			$selected[$key][$item] = 'selected';
		}
		$params = $_GET;
		
		?>
	<table>
	<tr>
		<td><div class="bold"><?php lang('mbs');?></div></td>
	</tr>
	<tr>
		<td>
			<table style="padding-left: 15px;">
				<tr>
                    <td><a href="?act=mbs.donate" class="<?php echo $selected[1]['donate']; ?>"><?php lang('donate')?></a></td>
                </tr>
		        <?php if ($selected[1]['donate']) {?>
                <tr>
                    <td>
                    <?php if (!empty($_GET['t'])) {
                        if( $this->transaction_info=$this->mbs->getBillInformation() ) {
	                    	echo $this->mbs->getStatusDescription($this->transaction_info['state'])."<br>\n";
	                        echo '<pre>';
	                        print_r($this->transaction_info);
	                        echo '</pre>';
                        } else {
                            echo $this->mbs->getError();
                        }
                    } else { ?>
	                    <form method="get">
	                        <input type="hidden" name="act" value="mbs.buy_redirect">
                            <?php if( strcasecmp($_SESSION['lang'], 'lv')==0 ): ?>
		                        <select name="price">
	                                <option value="0">0 LVL</option>
	                                <option value="5">0.05 LVL</option>
	                                <option value="10">0.1 LVL</option>
	                                <option value="20">0.2 LVL</option>
	                                <option value="30">0.3 LVL</option>
	                                <option value="40">0.4 LVL</option>
	                                <option value="50">0.5 LVL</option>
	                                <option value="100">1 LVL</option>
	                                <option value="150">1.5 LVL</option>
	                                <option value="200">2 LVL</option>
	                                <option value="250">2.5 LVL</option>
	                                <option value="300">3 LVL</option>
	                            </select>
	                            <input type="hidden" name="currency" value="LVL">
	                        <?php elseif( strcasecmp($_SESSION['lang'], 'et')==0 ): ?>
		                        <select name="price">
	                                <option value="0">0 EEK</option>
	                                <option value="5">5 EEK</option>
	                                <option value="5">7 EEK</option>
	                                <option value="5">10 EEK</option>
	                                <option value="5">15 EEK</option>
	                                <option value="5">18 EEK</option>
	                                <option value="5">25 EEK</option>
	                                <option value="5">32 EEK</option>
	                                <option value="5">45 EEK</option>
	                                <option value="5">50 EEK</option>
	                            </select>
	                            <input type="hidden" name="currency" value="EEK">
	                        <?php else: ?>
		                        <select name="price">
	                                <option value="0">0 LTL</option>
	                                <option value="30">0.3 LTL</option>
	                                <option value="50">0.5 LTL</option>
	                                <option value="60">0.6 LTL</option>
	                                <option value="70">0.7 LTL</option>
	                                <option value="100">1 LTL</option>
	                                <option value="150">1.5 LTL</option>
	                                <option value="170">1.7 LTL</option>
	                                <option value="200">2 LTL</option>
	                                <option value="300">3 LTL</option>
	                                <option value="400">4 LTL</option>
	                                <option value="500">5 LTL</option>
	                                <option value="600">6 LTL</option>
	                                <option value="700">7 LTL</option>
	                                <option value="800">8 LTL</option>
	                                <option value="900">9 LTL</option>
	                                <option value="1000">10 LTL</option>
	                            </select>
                                <input type="hidden" name="currency" value="LTL">
	                        <?php endif; ?>
	                        <input type="submit" name="submit" value="<?php lang('donate')?>" />
	                    </form>
	                <?php } ?>
                    </td>
                </tr>
                <?php } ?>
				<tr>
					<td><a href="?act=mbs.letters" class="<?php echo $selected[1]['letters']; ?>"><?php lang('buy_things')?></a></td>
				</tr>
				<?php if ($selected[1]['letters']) {?>
				<tr>
					<td>
					<?php
					$this->showElements();
					?>
					</td>
				</tr>
				<?php } ?>
				<?php if (!empty($this->data['bill_url'])) {?>
				<tr>
					<td><a href="<?php echo $this->data['bill_url'];?>"><?php echo $this->data['bill_url'];?></a></td>
				</tr>
				<?php }?>				
				<tr>
					<td><a href="?act=mbs.download" class="<?php echo $selected[1]['download']; ?>"><?php lang('download')?></a></td>
				</tr>
				<?php if ($selected[1]['download'] && !empty($this->transaction_info)) {?>
				<tr>
					<td>
					<?php 
						lang('show_code'); 
						echo $params[$this->params_item], ' : ', $this->items[$params[$this->params_item]]; ?> <br />

					<a href="?act=mbs.done&transaction_id=<?php echo $this->transaction_info['transaction_id']?>&<?php echo $this->params_item?>=<?php echo $params[$this->params_item]?>"><?php lang('done');?></a> | 
					<a href="?act=mbs.error&transaction_id=<?php echo $this->transaction_info['transaction_id']?>&<?php echo $this->params_item?>=<?php echo $params[$this->params_item]?>"><?php lang('error'); ?></a>
					</td>
				</tr>
				<?php } ?>
				<tr>
					<td><a href="?act=mbs.list" class="<?php echo $selected[1]['list']; ?>"><?php lang('transaction_list')?></a></td>
				</tr>
				<?php if ($selected[1]['list']) {?>
				<tr>
					<td>
					<?php 
						$this->showTransactions();
					?>
					<div></div>
					</td>
					
				</tr>
				<?php } ?>
			</table>
		</td>
	</tr>
	</table>		
		<?php
	}	
}
?>