<?php
/**
 * Example to show how to use SBBS Lib. 
 * Show 
 * 	- Menu
 *  - Register to subscription 
 *  - Unregister from subscription
 *  - Check user status
 *  
 * Example class
 * @package ExampleProject
 * @subpackage module
 * 
 * @author Liutauras Rickus <lr@vero.lt>
 *
 */

/**
 * ModuleSbbs
 * 
 * @package ExampleProject
 * @subpackage module
 * @author Liutauras Rickus <lr@vero.lt>
 */
class ModuleSbbs {
	
	/**
	 * Internal SBBS object
	 * @access protected
	 * @var SBBS
	 */
	var $sbbs = null;
	
	/**
	 * Internal Identify object
	 * @access protected
	 *
	 * @var Identify
	 */
	var $identify = null;
	
	/**
	 * This action
	 * @access protected
	 *
	 * @var array
	 */
	var $actions = array();
		
	/**
	 * Internal used data
	 * @access protected
	 *
	 * @var mixed
	 */
	var $data = '';
	
	/**
	 * Constructor
	 *
	 * @param SBBS $mbs - from lib
	 * @return ModuleMbs
	 */
	function ModuleSbbs( &$sbbs, &$identify ) {
		$this->sbbs = &$sbbs;
		$this->identify = &$identify;
		$this->actions = explode('.', $_REQUEST['act']);
	}
	
	/**
	 * Main action for module
	 *
	 */
	function parseAction() {
		if ($this->actions[0] != 'sbbs') {
			return false;
		}
		
		switch ($this->actions[1]) {
			// Register user by redirect
            case 'register_redirect':
            	$params = array(
                    'act'           => 'sbbs.register_back',
                    'session_id'    => session_id(),
                );
            	$this->sbbs->registerRedirect($params);
            	break;
            	
            // Unregister user by redirect
            case 'unregister_redirect':
            	$this->sbbs->unregisterRedirect();
            	break;
			
			// Register user by SOAP
			case 'register':
				// Check is user active if active do not register
				if (!$this->identify->isIdentified()) {
					$identify_url = $this->identify->MakeIdentify(session_id(), array('act' => 'sbbs.register'));
				}
				$account_id = $this->identify->getAccount();
				if ($this->sbbs->isActive($account_id)) {
					$this->data['user_active'] = true;
					break;
				}
				
				$params = array(
					'act' 			=> 'sbbs.register_back',
					'session_id'	=> session_id(),
					'trans' 		=> '%t',
					'op_id' 		=> '%o',
					'op_name' 		=> '%n',
					'service' 		=> '%s',
					'channel' 		=> '%c',
					'account' 		=> '%a',
				);
				
				$this->data['register_url'] = $this->sbbs->register($params, $_SESSION['lang'], null, date('Y-m-d H:i:s', strtotime("+1 MONTH")));
				break;
				
			// Unregister user by SOAP
			case 'unregister':
				$params = array();
				$this->data['unregister_url'] = $this->sbbs->unregister($params, $_SESSION['lang']);
				break;
				
			// Check user status in registration
			case 'check':
				if (!$this->identify->isIdentified()) {
					$identify_url = $this->identify->MakeIdentify(session_id(), array('act' => 'sbbs.check'));
				}
				$account_id = $this->identify->getAccount();
				if (empty($account_id)) {
					$this->data['info'] = 'User not identified: ';
				} else {
					$this->data['info'] = $this->sbbs->GetInfo($account_id);
				}
				break;
			default:
				
			// Show login form
			case 'login':
				// If user submits form
				if( !empty($_POST['login_submit']) ) {
					$login_phone = trim($_POST['phone']);
					$login_code = trim($_POST['code']);
					
					// Do not allow to login with empty code !!!
					if( !empty($login_phone) && !empty($login_code) ) {
						// We check if given MSISDN is active in service (with given login code as service ident)
						$info = $this->sbbs->getInfo($login_phone, 'phone', $login_code);
						if( $info['status']=='active' && $info['ident']==$login_code ) {
							$_SESSION['user'] = array(
								'logged'=> true,
								'info'	=> $info,
							);
						}
					}
					header('Location: ?act=sbbs.logged');
				}
				break;

			case 'logout':
				unset($_SESSION['user']);
				break;
			
			default:
		}		
		return true;
	}
	

	/**
	 * Main display for module. It must be change by template.
	 */
	function showBlock() {
		foreach ($this->actions as $key => $item) {
			$selected[$key][$item] = 'selected';
		}
		
		?>
	<table>
	<tr>
		<td><div class="bold"><?php lang('sbbs');?></div></td>
	</tr>
	<tr>
		<td>
			<table style="padding-left: 15px;">
				<!-- User zone with automatic login by redirecting to MBS -->
				<tr>
                    <td><a href="?act=sbbs.register_redirect" class="<?php echo $selected[1]['register_redirect']; ?>"><?php lang('register')?></a></td>
                </tr>
                <tr>
                    <td><a href="?act=sbbs.unregister_redirect" class="<?php echo $selected[1]['unregister_redirect']; ?>"><?php lang('unregister')?></a></td>
                </tr>
                <tr>
                    <td>
                    <?php if ($selected[1]['register_back'] && !empty($_GET['t'])) {
	                    if( $this->data=$this->sbbs->getRegisterInformation() ) {
	                        if( $this->data['subscriber_status']==MBS_SUBSCRIBER_STATUS_ACTIVE ) {
	                        	echo lang('user_active');
	                        }
	                        echo '<pre>';
	                        print_r($this->data);
	                        echo '</pre>';
                        } else {
                            echo $this->mbs->getError();
                        }
                    } ?>
                    </td>
                </tr>
				
				<tr>
                    <td><hr/></td>
                </tr>
				
				<!-- User zone with automatic login by currently browsing phone number -->
				<tr>
					<td><a href="?act=sbbs.register" class="<?php echo $selected[1]['register']; ?>"><?php lang('register')?></a></td>
				</tr>
				<?php 
				if ($this->data['user_active']) {
				?>
				<tr>
					<td><?php lang('user_active');?></td>
				</tr>
				<?php } ?>
				<?php if (!empty($this->data['register_url'])) {?>
				<tr>
					<td><a href="<?php echo $this->data['register_url'];?>"><?php echo $this->data['register_url'];?></a></td>
				</tr>
				<?php } ?>
				<tr>
					<td><a href="?act=sbbs.unregister" class="<?php echo $selected[1]['unregister']; ?>"><?php lang('unregister')?></a></td>
				</tr>
				<?php if (!empty($this->data['unregister_url'])) {?>
				<tr>
					<td><a href="<?php echo $this->data['unregister_url'];?>"><?php echo $this->data['unregister_url'];?></a></td>
				</tr>
				<?php }?>				
				<tr>
					<td><a href="?act=sbbs.check" class="<?php echo $selected[1]['check']; ?>"><?php lang('check_status')?></a></td>
				</tr>
				<?php if (!empty($this->data['info'])): ?>
				<tr>
					<td><pre>
					<?php print_r($this->data['info']) ?>
					</pre></td>
				<tr>
				<? endif; ?>
				
				<tr>
					<td><hr/></td>
				</tr>
				
				<!-- User zone with login form -->
				<tr>
					<td><a href="?act=sbbs.login" class="<?php echo $selected[1]['login']; ?>"><?php lang('login')?></a></td>
				</tr>
				<?php if( !empty($selected[1]['login']) && !$_SESSION['user']['logged'] ): ?>
				<tr><td>
					<form action="?act=sbbs.login" method="POST">
						<?php lang('login_phone')?>:<br/>
						<input name="phone" value="" /><br/>
						<?php lang('login_code')?>:<br/>
						<input name="code" value="" type="password" /><br/>
						<input name="login_submit" value="<?php lang('login')?>" type="submit" /><br/>
					</form>
				</td></tr>
				<?php endif; ?>
				<tr>
					<td><a href="?act=sbbs.logged" class="<?php echo $selected[1]['logged']; ?>"><?php lang('check_status')?></a></td>
				</tr>
				<?php if( !empty($selected[1]['logged']) ): ?>
				<tr>
					<td>
						<?php if( $_SESSION['user']['logged'] ): ?>
							<pre><?php print_r($_SESSION['user']['info']); ?></pre>
						<?php else: ?>
							<?php lang('login_error') ?>
						<?php endif;?>
					</td>
				</tr>
				<?php endif; ?>
				<tr>
					<td><a href="?act=sbbs.logout" class="<?php echo $selected[1]['logout']; ?>"><?php lang('logout')?></a></td>
				</tr>
			</table>
		</td>
	</tr>
	</table>		
		<?php
	}
	
}
?>