<?php
/**
 * Example to show how to use SBBSNotify Lib.
 * !!! Need write privileges to file 'users.log' or define file to constructor
 * Accepts notify from vero system and log to file
 * 
 * @package ExampleProject
 * @subpackage module
 * @author Liutauras Rickus <lr@vero.lt>
 *
 */

/**
 * ModuleSbbsNotify
 *
 * @package ExampleProject
 * @subpackage module
 * @author Liutauras Rickus <lr@vero.lt>
 */
class ModuleSbbsNotify {
	/**
	 * Internal SBBSNotify object
	 *
	 * @var SBBSNotify
	 */
	var $sbbs_notify = null;
	
	/**
	 * This action
	 *
	 * @var array
	 */
	var $actions = array();
	
	/**
	 * Log file name
	 *
	 * @var string
	 */
	var $file_name = 'users.log';
	
	/**
	 * Constructor
	 *
	 * @param SBBSNotify $sbbs_notify
	 * @param string $file_name - log file name default 'users.php'
	 * @return ModuleSbbsNotify
	 */
	function ModuleSbbsNotify( &$sbbs_notify, $file_name = '' ) {
		$this->sbbs_notify = &$sbbs_notify;
		$this->actions = explode('.', $_REQUEST['act']);
		
		if (!empty($file_name)) {
			$this->file_name = $file_name;
		}
		
	}
	
	/**
	 * Main action for module
	 */
	function parseAction() {
		
		if ($this->actions[0] != 'sbbs_notify') {
			return false;
		}
		// Get event data
		$notify_info = $this->sbbs_notify->getNotifyInfo();
		if( $error=$this->sbbs_notify->getError() ) {
			$this->sbbs_notify->showResponseError($error);
			die;
		}
		
		// Open log file
		$fp = @fopen($this->file_name, 'a');
		// If access denied repeat notify
		if ($fp===false) {
			$this->sbbs_notify->showResponseError('Access to file '.$this->file_name.' denied.');
			die;
		}
		
		// Process notify
		if( empty($notify_info) ) {
			$string = date('Y-m-d H:i:s')." | Got empty event\n";
			
			// Reponse that will be sent to MBS
			$response_state = false;
			$response_message = 'Event empty';
		} else {
			$string = $notify_info['event_date']." | user account_id: ".$notify_info['account_id'].' | Subscription service: ';
			$string .= $notify_info['subscribe_service_id']." | event occured: ".$notify_info['event'].' | description: ';
			$string .= $this->sbbs_notify->getEventDescription($notify_info['event'] )."\n";
			
			// Reponse that will be sent to MBS
			$response_state = true;
			$response_message = 'Event received';
		}
		fwrite($fp, $string);
		fclose($fp);
		
		// Send response to MBS
		$this->sbbs_notify->showResponse($response_state, $response_message);
		die;
	}
}
?>