<?php
/**
 * Lib purpose to simplify VERO MBS user bill process
 *
 * @package MBSLib
 * @author Liutauras Rickus <lr@vero.lt>
 */

/**
 * Debug levels
 */
defined('DEBUG_LEVEL_NONE') || define('DEBUG_LEVEL_NONE', 0);
defined('DEBUG_LEVEL_LOW') || define('DEBUG_LEVEL_LOW', 1);
defined('DEBUG_LEVEL_NORMAL') || define('DEBUG_LEVEL_NORMAL', 2);
defined('DEBUG_LEVEL_HIGH') || define('DEBUG_LEVEL_HIGH', 3);

/**
 * Debug types
 */
defined('DEBUG_TYPE_OUTPUT') || define('DEBUG_TYPE_OUTPUT', 0);
defined('DEBUG_TYPE_BUFFER') || define('DEBUG_TYPE_BUFFER', 1);

/**
 * WAP billing (MBS) constants
 */
defined('MBS_BILL_SOAP_VERSION') || define('MBS_BILL_SOAP_VERSION', '1.1');
if (!defined('MBS_BILL_SOAP_URL')) {
	if (MBS_BILL_SOAP_VERSION == '1.0') {
	   define('MBS_BILL_SOAP_URL', 'http://bill.vero.lt/soap/wap1.0');
	} else {
       define('MBS_BILL_SOAP_URL', 'http://bill.vero.lt/soap/wap1.1');
	}
}
defined('MBS_BILL_USER_REDIRECT_URL') || define('MBS_BILL_USER_REDIRECT_URL', 'http://bill.vero.lt/prepare');
defined('MBS_BILL_REDIRECT_URL') || define('MBS_BILL_REDIRECT_URL', 'http://bill.vero.lt/wap/bill');
/**
 * This constant describes how MBS sent WAP billing data is verified to be correct (for security reasons)
 * Posible values:  
 *  HIGH (default)  - High verification level (OpenSSL algorythm is used)
 *  LOW             - Lower verification level (SHA1 algorythm is used)
 *  NONE            - Verification is not performed
 */
defined('MBS_BILL_VERIFY_LEVEL') || define('MBS_BILL_VERIFY_LEVEL', 'HIGH');


/**
 * Subscription billing (SBBS) constants
 */
defined('MBS_SUBSCRIPTION_SOAP_URL') || define('MBS_SUBSCRIPTION_SOAP_URL', 'http://bill.vero.lt/soap/subscription');
defined('MBS_SUBSCRIPTION_REGISTER_URL') || define('MBS_SUBSCRIPTION_REGISTER_URL', 'http://bill.vero.lt/wap/register');
defined('MBS_SUBSCRIPTION_UNREGISTER_URL') || define('MBS_SUBSCRIPTION_UNREGISTER_URL', 'http://bill.vero.lt/wap/unregister');
/**
 * This constant describes how MBS sent WAP registration data is verified to be correct (for security reasons)
 * Posible values:  
 *  HIGH (default)  - High verification level (OpenSSL algorythm is used)
 *  LOW             - Lower verification level (SHA1 algorythm is used)
 *  NONE            - Verification is not performed
 */
defined('MBS_SUBSCRIPTION_VERIFY_LEVEL') || define('MBS_SUBSCRIPTION_VERIFY_LEVEL', 'HIGH');

/**
 * Subscription user status
 */
defined('MBS_SUBSCRIBER_STATUS_ACTIVE') || define('MBS_SUBSCRIBER_STATUS_ACTIVE', 'active');
defined('MBS_SUBSCRIBER_STATUS_SUSPENDED') || define('MBS_SUBSCRIBER_STATUS_SUSPENDED', 'suspend');
defined('MBS_SUBSCRIBER_STATUS_REMOVED') || define('MBS_SUBSCRIBER_STATUS_REMOVED', 'removed');


/**
 * Direct billing (Direct) constants
 */
defined('MBS_DIRECT_SOAP_URL') || define('MBS_DIRECT_SOAP_URL', 'http://bill.vero.lt/soap/direct');


/**
 * WEB billing (Bank) constants
 */
defined('MBS_BANK_URL') || define('MBS_BANK_URL', 'http://pay.vero.lt/bank');


/**
 * MBS identify constants
 * Identify URL's one for SOAP and one for user redirect
 */
defined('MBS_IDENTIFY_SOAP_URL') || define('MBS_IDENTIFY_SOAP_URL', 'http://bill.vero.lt/soap/account');
/**
 * Url to identify user
 */
defined('MBS_IDENTIFY_URL') || define('MBS_IDENTIFY_URL', 'http://bill.vero.lt/identify');
/**
 * This constant describes how MBS sent WAP identification data is verified to be correct (for security reasons)
 * Posible values:  
 *  HIGH (default)  - High verification level (OpenSSL algorythm is used)
 *  LOW             - Lower verification level (SHA1 algorythm is used)
 *  NONE            - Verification is not performed
 */
defined('MBS_IDENTIFY_VERIFY_LEVEL') || define('MBS_IDENTIFY_VERIFY_LEVEL', 'HIGH');
/**
 * This constant describes how users will be identified
 * Posible values:  
 * 	REDIRECT (default)	- User will be redirected to MBS and then return back with identification data
 *  REMOTE 				- Users will be identified with remote MBS SOAP function (works if operator turned on identification in your server)
 * 	BOTH				- User be tried to identify in both ways: REMOTE, then REDIRECT, if remote identitication failed
 */
defined('MBS_IDENTIFY_TYPE') || define('MBS_IDENTIFY_TYPE', 'REDIRECT');

/**
 * This constant describes which TRANSPORT type to use
 * Posible values: 
 * 	AUTO (default)  - check if are PHP5 SOAP extention use it other way use NuSOAP lib
 *  NUSOAP 			- use only nusoap library 
 * 	SOAP 			- use only PHP5 SOAP extention
 */
defined('MBS_TRANSPORT_TYPE') || define('MBS_TRANSPORT_TYPE', 'AUTO');

/**
 * Directory of MBS certificate files (used for high level security verifications)
 */
defined('MBS_CERTIFICATE_PATH') || define('MBS_CERTIFICATE_PATH', dirname(__FILE__).'/certificates/');

/**
 * MBSBase - tėvinė klasė, Debug, Error ir SOAP apdorojimas. Paveldi visos klasės.
 *
 * @package MBSLib
 * @author Liutauras Rickus <lr@vero.lt>
 *
 * @see Identify
 * @see MBS
 * @see SBBS
 * @see SBBSNotify
 *
 */
class MBSBase {

    /**
     * Saugomas URL, kur bus gražinamas vartotojas po Wap apmokestinimo, po prenumeratos Registracijos.
     *
     * @access protected
     *
     * @var string
     */
    var $back_url = '';
    
    /**
     * Saugomas slaptas žodis, kuris bus naudojamas saugumo parašui
     *
     * @access protected
     *
     * @var string
     */
    var $secret_key = '';

    /**
     *  Klaidos pranešimas
     *
     * @access protected
     *
     * @var string
     */
    var $error = '';

    /**
     * Pranešimų kaupimo lygis. Lygis pagal nutylėjimą {@link DEBUG_LEVEL_NORMAL}
     *
     * Galimos reikšmės:
     * # 0 - {@link DEBUG_LEVEL_NONE} nekaupiami pranešimai
     * # 1 - {@link DEBUG_LEVEL_LOW} kaupiami tik klaidų pranešimai
     * # 2 - {@link DEBUG_LEVEL_NORMAL} kaupiami tik reikalingi pranešimai
     * # 3 - {@link DEBUG_LEVEL_HIGH} kaupiami visi pranešimai
     *
     * @access protected
     *
     * @var int
     */
    var $debug_level = DEBUG_LEVEL_NORMAL;

    /**
     * Debug informacijos išvedimo tipas. Pagal nutylėjimą {@link DEBUG_TYPE_OUTPUT}
     *
     * Galimos reikšmės:
     * # 0 - {@link DEBUG_TYPE_OUTPUT} išvedimas į išvedimo įrenginį
     * # 1 - {@link DEBUG_TYPE_BUFFER} kaupiamas buferyje
     *
     * @access protected
     *
     * @var int
     */
    var $debug_type = DEBUG_TYPE_BUFFER;

    /**
     * Laikoma Debug informacija
     *
     * @access protected
     *
     * @var string
     */
    var $debug_buffer = '';

    /**
     * Naudojame SOAP klasė
     *
     * Galimos reikšmės:
     *  # NuSOAP - skirtas php4 {@link http://sourceforge.net/projects/nusoap/}
     *  # SOAP - skirtas php5 soap išplėtimui {@link http://www.php.net/soap}
     *
     * @access protected
     *
     * @var string NuSOAP|SOAP
     */
    var $soap_type = null;

    /**
     * SOAP užklausų kreipinių adresas
     * 
     * @access protected
     *
     * @var string URL
     */
    var $soap_url = '';

    /**
    * Sesijos saugomo parametro priešdėlis.
    *
    * Pvz.: Identify identifikuoto vartotojo duomenis saugo sesijos kintamajame „identify_$suffix“.
    *
    * @access protected
    * @var string
    */
    var $suffix = '_class_data';

    /**
    * Ar saugoti reikiamus duomenis sesijoje
    * @access protected
    *
    * @var boolean
    */
    var $use_session = true;

    /**
     * Konstruktorius, neatliekantis jokiu konkrečių veiksmų
     *
     * @return MBSBase
     */
    function MBSBase() {}

    /**
     * Nustatoma naudojama SOAP klasė ir SOAP adresui priskiriama paduota reikšmė $url
     * 
     * @param string $url URL
     *
     * @see soap_type
     */
    function initSoap($url) {
        $this->soap_url = $url;

        $this->debug('URL for SOAP '.$this->soap_url, DEBUG_LEVEL_NORMAL );

    	if (in_array( strtoupper(MBS_TRANSPORT_TYPE), array('NUSOAP', 'SOAP') )) {
		    $this->debug('SOAP SET TO '.MBS_TRANSPORT_TYPE, DEBUG_LEVEL_NORMAL );
    		$this->soap_type = MBS_TRANSPORT_TYPE;
    	} else {
		    $this->debug('Detecting SOAP lib...', DEBUG_LEVEL_NORMAL );
    		if (!extension_loaded('soap')) {
				$this->soap_type = 'NuSOAP';
		    	$this->debug('NuSOAP client used for SOAP implementation', DEBUG_LEVEL_NORMAL );
	    	} else {
				$this->soap_type = 'SOAP';
		    	$this->debug('Using PHP SOAP implementation', DEBUG_LEVEL_NORMAL );
	    	}
    	}
    	
    	if (strtoupper($this->soap_type) == 'NUSOAP') {
			require_once(dirname(__FILE__).'/nusoap/nusoap.php');
    	}
    }

    /**
     * Kaupiame debug informaciją $msg jeigu klasės savybės $debug_level yra mažesnis arba lygus $need_level.
     * 
     * Jeigu $this->debug_type nustatytas kaip {@link DEBUG_TYPE_OUTPUT}, debug informacija vedama į ekrana, kitu atveju į buferį.
     *
     * @param string $msg Žinutės tekstas
     * @param int $need_level Žinutės debug lygis
     * 
     * @see getDebug
     *
     */
    function debug($msg, $need_level) {
        $msg = htmlspecialchars($msg);
        if ($need_level <= $this->debug_level) {
            $this->debug_buffer .= $msg."<br />\n";
            if ( $this->debug_type == DEBUG_TYPE_OUTPUT) {
                echo $msg."<br />\n";
            }
        }
    }

    /**
     * Nustatomas debug kaupimo lygis į $level ir kaupimo tipas į $type.
     *
     * @param int $level
     * @param int $type
     *
     * @see getDebug
     * @see debug_type
     * @see debug_level
     */
    function setDebug( $level, $type=DEBUG_TYPE_BUFFER ) {
        $this->debug_level = $level;
        $this->debug_type = $type;

        $this->debug('Debug level set to '.$this->debug_level.'. Debug type set to: '.$this->debug_type, DEBUG_LEVEL_HIGH );
    }

    /**
     * Gražinama sukaupta debug informacija. 
     *
     * @return string
     */
    function getDebug() {
        return $this->debug_buffer;
    }

    /**
     * Nustatomas klaidos pranešimas su debug lygiu {@link DEBUG_LEVEL_LOW}
     *
     * @param string $msg
     */
    function setError( $msg ) {
        $this->error .= $msg."\n";
        $this->debug('Error: '.$msg, DEBUG_LEVEL_LOW );
    }

    /**
     * Grąžinami klaidų pranešimai
     *
     * @return string
     */
    function getError() {
        return $this->error;
    }

    /**
     * Gražinama ar įvyko klaida
     *
     * @return boolean
     */
    function isError() {
        return (!empty($this->error) ? true : false);
    }

    /**
     * Paruošiami parametrai pagal "JArray" tipą (naudojamas soap metoduose), kiekvienam masyvo $params elementui
     *
     * Jeigu $params atrodo:
     * <code>
     * $params = array(
     *    'key1'=>'value1',
     *    'key2'=>'value2',
     * );
     * </code>
     * Tai metods grazins:
     * <code>
     * array(
     *  array(
     *    'name'=>'key1',
     *    'value'=>'value1',
     *  ),
     *  array(
     *    'name'=>'key2',
     *    'value'=>'value2',
     *  )
     * )
     * </code>
     *
     * @param array $params
     * @access protected
     * @return array
     */
    function _prepareParams($params) {
        $return = array();
        foreach ($params as $key => $item) {
            $return[] = array( 'name' => $key, 'value' => $item );
        }

        return $return;
    }

    /**
     * Paruošia URL parametrus. Jeigu $this->back_url nustatytas, tuomet visi šie parametrais sudedami į $this->back_url
     * 
     * Taip pat nuorodoje galima naudoti žymenis:
     *  # %t arba %{transaction_id}% - Tranzakcijos identifikatorius
     *  # %s arba %{service_id}% - Paslaugos identifikatorius
     *  # %c arba %{channel_id}% - Kanalo identifikatorius
     *  # %o arba %{operator_id}% - Operatoriaus identifikatorius
     *  # %a arba %{account_id}% - Vartotojo identifiaktorius
     * 	# %n arba %{operator_name}% - Operatoriaus pavadinimas (pvz.. bite_lt, tele2_lv, t.t.)
     *
     * Pavyzdys:
     *   Jeigu $this->back_url nustatom <b>http://example.lt?trans=%t&s=%s&service=%{service_id}%&a=test</b>
     *  , o tranzakcijos ID yra <b>2456</b> ir paslaugos ( Service ) ID yra <b>1</b>
     *   Vartotojas bus gražintas į URL : <b>http://example.lt?trans=2456&s=1&service=1&a=test</b>
     *
     * @param array $url_params URL parametrai kuriuos norite perduoti
     * @return array Parametrai ( masyvas ) kurį reikia naudoti kuriant tranzakciją
     *
     * @see setBackUrl
     */
    function prepareParams($options) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );
        
        // If back URL is asked
        if ( !empty($this->back_url) ) {
        	$this->debug('Back URL: '.$this->back_url, DEBUG_LEVEL_HIGH );
        	$options['url'] = $this->formatUrl($this->back_url, $options, false);
        	$this->debug('URL constructed '.$options['url'], DEBUG_LEVEL_NORMAL);
        }

        return $this->_prepareParams($options);
    }

    /**
     * SOAP metodo kvietimas. Naudojantis nustatyta soap klasė
     *
     * @param string $function Soap metodo pavadinimas
     * @param array $soap_params Metodo parametrai pagal struktura (parameter_name => parameter_value)
     * 
     * @access protected
     *
     * @return array|boolean Sėkmingo kvietimo atveju gražinamas array, nesėkmės atvėju false
     *
     * @see soap_type
     */
    function _SOAPCall($function, $soap_params) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );

        $method_name = "_Call$this->soap_type";
        $return = $this->$method_name($function, $soap_params);

        return $return;
    }

    /**
     * SOAP metodo kvietimas naudojantis PHP SOAP moduliu.
     *
     * @param string $function Soap metodo pavadinimas
     * @param array $soap_params Metodo parametrai pagal struktura (parameter_name => parameter_value)
     * @access protected
     *
     * @return array|boolean Sėkmės atvėju gražinamas array, nesėkmės false
     * 
     * @link http://www.php.net/soap
     */
    function _CallSOAP($function, $soap_params) {

        // Initiating SOAP client:
        $client = new SoapClient( $this->soap_url.'?wsdl', array('trace' => 1) );

        $this->debug('Call soap function: '.$function.' with parameters: '.var_export($soap_params, true), DEBUG_LEVEL_NORMAL);
        // Performing request to create transaction
        $return = $client->__soapCall($function, $soap_params);
        $this->debug('SOAP request Headers: '.var_export($client->__getLastResponseHeaders(), true), DEBUG_LEVEL_HIGH);
        $this->debug('SOAP request: '.var_export($client->__getLastRequest(), true), DEBUG_LEVEL_HIGH);

        $this->debug('SOAP response Headers: '.var_export($client->__getLastResponseHeaders(), true), DEBUG_LEVEL_HIGH);
        $this->debug('SOAP response: '.var_export($client->__getLastResponse(), true), DEBUG_LEVEL_HIGH);
        $this->debug('Function return: '.var_export($return, true), DEBUG_LEVEL_NORMAL);

        return $return;
    }

    /**
     * SOAP metodo kvietimas naudojantis NuSOAP biblioteka
     *
     * @param string $function Soap metodo pavadinimas
     * @param array $soap_params Metodo parametrai pagal struktura (parameter_name => parameter_value)
     * @access protected
     *
     * @return array|boolean Sėkmės atvėju gražinamas array, nesėkmės false
     * 
     * @link http://sourceforge.net/projects/nusoap/
     */
    function _CallNuSOAP($function, $soap_params) {
        // Initiating SOAP client:
        $client = new nusoap_client( $this->soap_url );

        // Checking for error
        if( $error=$client->getError() ) {
            $this->setError('Cat not reach SOAP server: '.$error);
            return false;
        }

        $this->debug('Call soap function: '.$function.' with parameters: '.var_export($soap_params, true), DEBUG_LEVEL_NORMAL);
        // Performing request to create transaction
        $return = $client->call($function, $soap_params);
        $this->debug('SOAP request: '.var_export($client->request, true), DEBUG_LEVEL_HIGH);

        $this->debug('SOAP response: '.var_export($client->response, true), DEBUG_LEVEL_HIGH);
        $this->debug('Function return: '.var_export($return, true), DEBUG_LEVEL_NORMAL);

        // Checking for error
        if( $error=$client->getError() ) {
            $this->setError('SOAP return error: '.$error.': '.$return['detail'] );
            return false;
        }

        return $return;
    }

    /**
     * Nustatoma ar saugoti duomenis sesijoje
     *
     * @param boolean $isUse
     */
    function setSessionUse( $isUse = true ) {
        $this->use_session = (boolean)$isUse;
    }

    /**
     * Nustatomas gražinimo url. Jis gali būti bet koks. Jeigu yra galimybė iš Vero(MBS) platformos bus bandom nukreipti vartotoją į šią nuorodą. Nuorodoje galima naudoti parametrus, gražinant vartotoją į nuorodą bus pakeisti reikšmėmis.
     *
     * Galimi parametrai:
     * # %t arba %{transaction_id}% - pakeistas į naudojamos transakcijos ID
     * # %s or %{service_id}% - pakeistas į naudojamos paslaugos(service) ID
     * # %c or %{channel_id}%- pakeistas į naudojamo kanalo (channel) ID
     * # %o or %{operator_id}%- pakeistas į vartotojo operatoriaus (operator) ID
     * # %a or %{account_id}%- pakeistas į vartotojo identifikatorių (account) ID
     * # %n or %{operator_name} - pakeistas į vartotojo operatoriaus pavadinimą (pvz.: bite_lt, tele2_lv, ect.)
     *
     * Didžiosios mažosios raidės neturi reikšmės.
     *
     * @param string $url URL
     */
    function setBackUrl($url) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );

        $this->back_url = $url;
        $this->debug('Back url is set to :'.$this->back_url, DEBUG_LEVEL_HIGH );
    }
    
    /**
     * Nustatomas slaptas žodis, kuris bus naudojamas saugumo parašui
     *
     * @param string $secret_key Slapto žodžio reikšmė
     */
    function setSecretKey($secret_key) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH);

        $this->secret_key = $secret_key;
        $this->debug('Secret key is is set to :'.$this->secret_key, DEBUG_LEVEL_HIGH);
    }
    
    
    //********** Pagalbiniai metodai (URL formavimas ir kt.) **********//
    
    /**
     * Grąžina "suklijuotus" URL parametrus
     * 
     * @param string $url URL prie kurio lipdysime parametrus
     * @param array $url_params URL parametru masyvas
     * @param boolean $url_encode (optional) Flag if URL encoding should be performed
     * 
     * @return string URL eilute su pridetais parametrais
     */ 
    public function formatUrl($url, $url_params, $url_encode=true) {
        $url_parts = array();
        foreach($url_params as $param_name => $param_value) {
            $param_value = $url_encode ? urlencode($param_value) : $param_value;
        	$url_parts[] = $param_name.'='.$param_value;
        }
        if( strpos($url, '?')===false ) {
            return $url.'?'.implode('&', $url_parts);
        } else {
            return $url.'&'.implode('&', $url_parts);
        }
    }
    
    
    
    //********** Pasirasymo metodai **********//
    
    /**
     * Pasirašo pateiktus parametrus žemo patikimumo parašu
     *
     * @param array $params Masyvas su parametrais
     * 
     * @return array Pasirašytas masyvas
     */
    function signArrayLow($params) {
        $params['s1'] = sha1(implode('', $params).$this->secret_key);
        return $params;
    }

    /**
     * Pasirašo pateiktą URL žemo patikimumo parašu
     *
     * @param string $url URL, kurį reikia pasirašyti
     * 
     * @return string Pasirašytas URL
     */    
    function signUrlLow($url) {
    	// Pasiimam URL parametrus
    	$params = array();
    	$query_string = parse_url($url, PHP_URL_QUERY);
    	parse_str($query_string, $params);

    	// Sugeneruojamas parasas
    	$params = $this->signArrayLow($params);

    	// Sugeneruojama naujas URL
    	$base_url = str_replace('?'.$query_string, '', $url);
    	return $base_url . '?' . http_build_query($params);
    }


        
    //********** MBS paraso tikrinimo metodai metodai **********//
    
    /**
     * Patikrina ar parametrai teisingai pasirašyti žemu patikimumu
     *
     * @param array $params Masyvas su parametrais
     * 
     * @return boolean Ar masyvas pasirašytas teisingai
     */    
    function verifyArrayLow($params) {
    	$signature = isset($params['s1']) ? $params['s1'] : '';
    	unset($params['s1'], $params['s2']);

    	return (sha1(implode('', $params).$this->secret_key) == $signature);
    }
    
	/**
	 * Patikrina ar parametrai teisingai pasirašyti aukštu patikimumu
	 *
	 * @param array $params Masyvas su parametrais
	 * 
	 * @return boolean Ar masyvas pasirašytas teisingai
	 */
	function verifyArrayHigh($params) {
        $signature = isset($params['s2']) ? $params['s2'] : '';
		unset($params['s1'], $params['s2']);

		// Patikrina perduota parasa
		$pub_key = file_get_contents(MBS_CERTIFICATE_PATH . 'vero.pub');
		$pkeyid = openssl_get_publickey($pub_key);

		// verify signature
		$ok = openssl_verify(implode('', $params), base64_decode($signature), $pkeyid);
		// free the key from memory
		openssl_free_key($pkeyid);

		return $ok;
	}
    
	/**
	 * Patikrina ar URL turi teisingą žemo patikimumo parašą
	 * 
	 * @param string $url URL, kurį reikia tikrinti
	 * 
	 * @return boolean Ar URL pasirašytas teisingai
	 */
    function verifyUrlLow($url) {
        // Pasiimam URL parametrus
        $params = array();
        $query_string = parse_url($url, PHP_URL_QUERY);
        parse_str($query_string, $params);
    
        return $this->verifyArrayLow($params);
    }

    /**
     * Patikrina ar URL turi teisingą aukšto patikimumo parašą
     * 
     * @param string $url URL, kurį reikia tikrinti
     * 
     * @return boolean Ar URL pasirašytas teisingai
     */    
    function verifyUrlHigh($url) {
    	// Pasiimam URL parametrus
        $params = array();
        $query_string = parse_url($url, PHP_URL_QUERY);
        parse_str($query_string, $params);
        
    	return $this->verifyArrayHigh($params);
    }
    
    /**
     * Pagal tranzakcijos state reikšmę grąžiną state aprašymą.
     *
     * Tranzakcijų state galimos reikšmės:
     *  # <b>open</b> – Sukurta tranzakcija. Laukiami tolimesni veiksmai.
     *  # <b>ac_unknown</b> – MBS sistemi nepavyko identifikuoti vartotojo
     *  # <b>ac_ready</b> – vartotojas identifikuotas ir paruoštas apmokestinimui
     *  # <b>ac_block</b> – vartotojas negali būti apmokestinamas
     *  # <b>ac_limit</b> – vartotojas pasiekė mėnesio limitą ir negali būti apmokestinamas.
     *  # <b>ac_cancel</b> – vartotojas atsisakė mokėti. Paspaudė mygtuką „Ne“ arba „Atgal“
     *  # <b>ac_done</b> – vartotojas sutiko būti apmokestinamas pradedamas apmokestinimo procesas
     *  # <b>op_wait</b> – apmokestinimas pradėtas laukiame apmokestinimo rezultato iš operatoriaus. Omnitel, LMT operatorių atvejais.
     *  # <b>cancel_config</b> – neteisingai sukonfigūruota paslauga.
     *  # <b>cancel</b> – atšauktas dėl MBS vidinių problemų
     *  # <b>op_cancel</b> – operatorius atšaukė apmokestinimą (ši būsena suteikiamas visosm vidinėms klaidosm)
     *  # <b>op_cancel_connect</b> – nepavyko prisijungti prie operatoriaus
     *  # <b>op_cancel_data</b> – neteisingai pateikiami parametrai operatoriaus sistemai
     *  # <b>op_cancel_config</b> – klaida parametruose
     *  # <b>op_cancel_param</b> – neįmanoma apmokestinti už nurodytą kainą ar valiutą
     *  # <b>op_cancel_unavail</b> – nepasiekiama operatoriaus sistema
     *  # <b>op_timeout</b> – įvyko timeout į operatoriaus sistemą
     *  # <b>op_ac_blocked</b> – operatorius nebeleidžia apmokestinti vartotojo
     *  # <b>op_ac_limit</b> – vartotojas pasiekė mėnesio limitą operatoriaus sistemoje
     *  # <b>op_done</b> – sėkmingai įvykęs apmokestinimas
     *  # <b>rollback</b> – apmokestinimas atšauktas vartotojui pinigai grąžinti. Šį veiksmą inicijuoja partnerio sistema.
     *  # <b>commit</b> – pilnai pabaigtas apmokestinimas. Vartotojas sėkmingai apmokestintas ir tranzakcija nebekeičiama.
     * 
     * @param string $state Tranzakcijos būsena
     * 
     * @return string Tranzakcijos būsenos aprašymas
     */
    function getStatusDescription($state) {
        $description_array = array(
            'open'          => 'Just created transaction. Waiting for user to come for billing',
            'ac_unknown'    => 'MBS failed to identify user which came for billing;',
            'ac_ready'      => 'MBS successfully identified user which came for billing;',
            'ac_block'      => 'User is blocked for billings in MBS;',
            'ac_limit'      => 'User has exceeded it\'s monthly limit, he is blocked to spend more money in MBS billings that month;',
            'ac_cancel'     => 'User refuses to pay after agreement question;',
            'ac_done'       => 'User agreed to pay after agreement question;',
            'op_wait'       => 'User billing started, waiting for result;',
            'cancel_config' => 'Billing canceled because of wrong configuration of billing parameters;',
            'cancel'        => 'Billing canceled because of any other error in MBS;',
            'op_cancel'     => 'Operator canceled billing.',
            'op_cancel_connect'     => 'MBS can\'t connect to operator platform',
            'op_cancel_data'        => 'Bad formated date pase to operator',
            'op_cancel_config'      => 'Error: in service configuration',
            'op_cancel_param'       => 'Can\'t bill for such price',
            'op_cancel_unavail'     => 'Operator server unreachable.',
            'op_timeout'            => 'Connection to operator server timeout',
            'op_ac_blocked' => 'User account was blocked at operator;',
            'op_ac_limit'   => 'User account money was not enough to pay required amount of money;',
            'op_done'       => 'Billing successfully ended at operator billing system;',
            'rollback'      => 'Successful billing was canceled after a while and money returned back to user. In most cases reason is that user failed to receive service he/she paid for);',
            'commit'        => 'This state marks completely finished all billing steps and user received service he/she paid for;',
        );
        
        return (empty($description_array[$state]) ? 'Status '.$state. 'not found' : $description_array[$state] );
    }

}

?>