<?php
/**
 * Lib purpose to simplify acceptance of MBS notify messages
 *
 * @package MBSLib
 * 
 * @author Valdas Petrulis <vpe@vero.lt>
 */

require_once('MBSBase.class.php');


/**
 * Klasė skirta priimti sistema-sistema tipo pranešimus apie kažkurią iš MBS paslaugų.
 * Ši klasė yra tėvinė klasė, kurią naudoja kiti sisteminių žinučių priėmėjai: SBBSNotify
 *
 * Pranešimai yra siunčiami ir duomenis perduodami POST metodu. Apdorojus pranešimą gražinamas atsakymas atspausdinant jį. Galimos gražinamos reikšmės:
 * TRUE – pranešimas sėkmingai apdorodas ir priimtas. Pranešimo nekartoti
 * FALSE – pranešimas apdorotas, tačiau nepriimtas, nes nekorektiškas. Pranešimo nekartoti
 * ERROR – nepavyko priimti pranešimo. Pranešimą kartoti (šį statusą reiškia ir bet koks kitas atsakymas iš pranešimus priimančio URL)
 * 
 * Norimą atsakymą reikia suformuoti su metodais showResponse() ir showResponseError().
 * 
 * @see MBSNotifyBase::showResponse()
 * @see MBSNotifyBase::showResponseError()
 *
 * @package MBSLib
 * @author Valdas Petrulis <vpe@vero.lt>
 */
class MBSNotifyBase extends MBSBase {
	
    /**
     * Paslaugos identifikatorius
     *
     * @access protected
     * 
     * @var int
     */
    var $service_id;

    /**
     * Pranešimo duomenys naudojami vidinėms reikmėms.
     * 
     * @access protected
     *
     * @var array
     */
    var $data;

    /**
     * MBSNotifyBase klasės konstruktorius
     *
     * @param int $service_id Paslaugos ID
     * @param string $secret_key Paslaugos slaptas raktas
     *
     * @return MBSNotifyBase
     */
    function MBSNotifyBase($service_id, $secret_key) {
        parent::MBSBase();
    	
        if (empty($service_id )) {
            $this->setError("Service ID not set");
        } else {
        	$this->service_id = (int)$service_id;
    	}

        if (empty($secret_key )) {
            $this->setError("Service secret key (password) not set");
        } else {
        	$this->setSecretKey($this->secret_key);
        }
    }

    /**
     * Gražinama pranešimo informacija, pranešimo struktūra skirtinga kiekvienai paslaugai
     * 
     * @return array Jeigu informacijos nepavyko gauti bus gražintas tusčias masyvas
     */
    function getNotifyInfo() {
        if( $this->verifyArrayHigh($_REQUEST) ) {
        	$this->data = $_REQUEST;
        } else {
        	$this->setError('Authorization failed');
        }
    }

    /**
     * Atsakymas į pranešimą, kad daugiau nebesiųstu pranešimų.
     *
     * Jeigu pavyko autorizuoti pranešimą išvedamas „TRUE $msg“, jei klaida „FALSE $msg“.
     * Po šio atsakymo pranešimas nebus kartojamas.
     *
     * @param string $is_good Žymė ar pranešimas buvo tinkamai priimtas į partnerio sistemą (true), ar netiko partnerio sistemai (false)
     * @param string $message Papildomas tekstas prie atsakymo (pvz., pranešimo netinkamumo apibūdinimas, "User does not exist")
     */
    function showResponse($is_good = true, $message = '') {
        $notify = $this->getNotifyInfo();
        if ( !$this->isError() && $is_good ) {
            echo 'TRUE;'.$message;
        } else {
            echo 'FALSE;'.$message;
        }
    }

    /**
     * Atsakymas sistemai, kad įvyko sistemos klaida ir pranešimą reikia pakartoti.
     *
     * Pranešimas bus pakartotas jeigu ir nieko nebus gražinama. Išvedama „ERROR $msg“.
     *
     * @param string $message Papildomas tekstas prie atsakymo (pvz., klaidos apibūdinimas, "Failed to coneect to DB")
     */
    function showResponseError($message = '') {
        echo 'ERROR;'.$message;
    }
    
}
?>