<?php
/**
 * Lib purpose to simplify accept Subscription Notify messages from SBBS (Subscription Based Billing System)
 *
 * @package MBSLib
 * 
 * @author Liutauras Rickus <lr@vero.lt>
 * @author Valdas Petrulis <vpe@vero.lt>
 */

require_once('MBSNotifyBase.class.php');


/**
 * Klasė skirta priimti prenumeratinės paslaugos pranešimus apie vartotojų statusų pasikeitimus.
 * Reikia naudoti SBBS_SUBSCRIPTION_SERVICE_ID
 * 
 * @see MBSNotifyBase
 *
 * @package MBSLib
 * 
 * @author Liutauras Rickus <lr@vero.lt>
 * @author Valdas Petrulis <vpe@vero.lt>
 */
class SBBSNotify extends MBSNotifyBase {

    /**
     * SBBSNotify klasės konstruktorius
     *
     * @param int $service_id Prenumeratinės paslaugos ID (SBBS_SUBSCRIPTION_SERVICE_ID)
     * @param string $secret_key Paslaugos slaptas raktas
     *
     * @return SBBSNotify
     */
    function SBBSNotify($service_id, $secret_key) {
        parent::MBSNotifyBase($service_id, $secret_key);
    }

    /**
     * Gražinama pranešimo informacija.
     *
     * Pranešimo struktūra yra:
     * <code>
     * array(
     *  'partner_id' => Partnerio identifikatorius Vero sistemoje
     *  'subscribe_service_id' => Prenumeratinės paslaugos ID
     *  'account_id' => Vartotojo identifikatorius Vero sistemoje
     *  'event' => Pranešimo įvykis plačiau getEventDescription
     *  'event_date' => Įvykio data
     *  'secret_check' => Pranešimo parašas, kurį reikia tikrinti
     * )
     * </code>
     * 
     * @return array Jeigu informacijos nepavyko gauti bus gražintas tusčias masyvas
     */
    function getNotifyInfo() {
        $notify_info = parent::getNotifyInfo();
        
        // Check if service ID same as in received variable 'subscribe_service_id'
        if( $notify_info['subscribe_service_id']==$this->service_id ) {
        	return $notify_info;
        } else {
        	$this->setError('Service not same');
        }
    }

    /**
     * Gražinamas įvykio aprašymas anglų kalba.
     *
     * Galimos $event_name reikšmės:
     *  # register – Registruotas naujas vartotojas
     *  # renew – Vartotojo prenumerata pratęsta
     *  # resume – Vartotojo prenumerata vėl pratęsta, po prieš tai vykysio sustabdymo
     *  # suspend – Nepavyko pratęsti prenumeratos, vartotojo narystė sustabdyta
     *  # unregister – Vartotojas pašalintas iš prenumeratos
     *  # subscribe – Vartotojas pilnai registruotas į operatoriaus sistemą
     *  # unsubscribe – Vartotojas pilnai išregistruotas iš operatoriaus sistemos
     * 
     * @param string $status Statuso pavadinimas
     *
     * @return string Statuso aprašymas
     */
    function getEventDescription($event_name) {
    	$description_array = array(
    		// Statistical events, if billing was done
			'register' 		=> "New user registered",
			'renew' 		=> "User subscription renewed",
			'resume' 		=> "User subscription renewed again, after previous suspension",

    		// Technical events, if no billing done
			'suspend' 		=> "Can't renew subscription. User status changed to suspended",
			'unregister' 	=> "User unregistered",

    		// Advanced usage events
			'subscribe' 	=> 'Subscribing user to operator system',
			'unsubscribe' 	=> 'Unsubscribing user from operator system',
    	);

    	return (empty($description_array[$event_name]) ? 'Event "'.$event_name.'" not found' : $description_array[$event_name]);
    }

}
?>